/**
 * CMSC435 Group Project “Authentication”
 * For this app we used the following resources
 *  :https://developer.android.com/samples/FingerprintDialog/index.html
 *  :https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/
 */

/**
 * This activity represents the fingerprint authentication system. When this activity starts, the
 * user will be asked to scan their fingerprint on the device’s fingerprint scanner.
 * If the authentication system is a success, then the user will be redirected to a survey page,
 * otherwise, the user will not be able to login.
 */
package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.Manifest;
import android.app.Activity;
import android.app.KeyguardManager;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.hardware.fingerprint.FingerprintManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.Settings;
import android.security.keystore.KeyGenParameterSpec;
import android.security.keystore.KeyPermanentlyInvalidatedException;
import android.security.keystore.KeyProperties;
import android.support.v4.app.ActivityCompat;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLEncoder;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.net.ssl.HttpsURLConnection;

import static java.lang.Thread.sleep;

public class FingerPrintAuth extends Activity {


    private static final String TAG = "FingerPrintAuth ";

    private boolean loggedIn = false;

    private KeyStore keyStore;
    private KeyGenerator keyGenerator;
    private Cipher cipher;
    static final String DEFAULT_KEY_NAME = "default_key";
    private FingerprintManager.CryptoObject cryptoObject;
    private FingerprintHelper helper;

    private TextView error_message, fingerprint_instruct;
    private ImageView fingerPrintImage;

    private FingerprintManager fingerprintManager;

    private boolean isListenerCreated = false;

    private Button security_setting_button, restart_button;

    //The time this activity got started.
    public static long timeStarted;

    //Total time taken
    public static long TOTAL_TIME_TAKEN;

    //The time from the start to the first action
    public static long firstActionTime;

    public static boolean isFirstAction = true;

    //Holds the time for the most current action
    public static long timeForTheMostCurrentAction;

    //The number of failed authentication
    public static int numFailedAuthentication;

    //The number of successful authentication
    public static int numSuccessAuthentication;

    //number of fingerScan
    public static int TOTAL_CLICKS = 0;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.finger_print_layout);


        timeStarted = System.currentTimeMillis();

        security_setting_button = (Button) findViewById(R.id.security_setting_button);
        restart_button = (Button) findViewById(R.id.restart_button);
        security_setting_button.setVisibility(View.GONE);
        restart_button.setVisibility(View.GONE);
        error_message = (TextView) findViewById(R.id.error_message);
        fingerprint_instruct = (TextView) findViewById(R.id.fingerprint_instruct);
        fingerPrintImage = (ImageView) findViewById(R.id.fingerprintImage);
        fingerPrintImage.setImageResource(R.drawable.ic_fingerprint_red);

        fingerprintManager = getSystemService(FingerprintManager.class);

        KeyguardManager keyguardManager = getSystemService(KeyguardManager.class);

        //Check hardware and system settings for fingerprint system
        if (!fingerprintManager.hasEnrolledFingerprints()) {
            //Tell the user to register a fingerprint
            error_message.setText(R.string.fingerprint_register_fingerprint);
            fingerPrintImage.setVisibility(View.GONE);
            fingerprint_instruct.setVisibility(View.GONE);
            security_setting_button.setVisibility(View.VISIBLE);
            restart_button.setVisibility(View.VISIBLE);
        } else if (!keyguardManager.isKeyguardSecure()) {
            //Tell the user to secure lock screen
            error_message.setText(R.string.fingerprint_lock_screen_error);
            fingerPrintImage.setVisibility(View.GONE);
            fingerprint_instruct.setVisibility(View.GONE);
            security_setting_button.setVisibility(View.VISIBLE);
            restart_button.setVisibility(View.VISIBLE);
        } else {

            try {

                generateKey();
            } catch (FingerPrintAuth.FingerprintException e) {
                e.printStackTrace();
            }

            if (initCipher()) {
                cryptoObject = new FingerprintManager.CryptoObject(cipher);
                boolean isLast = getIntent().getExtras().getBoolean(MainActivity.IS_LAST);
                long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

                helper = new FingerprintHelper(fingerprintManager, getApplicationContext(),
                        FingerPrintAuth.this, this, isLast, sessionID
                , this);
                try {
                    sleep(500);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                helper.startListening(cryptoObject);

                isListenerCreated = true;
            }

        }
    }

    @Override
    public void onStop() {
        super.onStop();
        Log.i(TAG, " on stop ");
        if (helper != null) {
            helper.stopListening();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        Log.i(TAG, " on Resume " + "helper is null : "+ (helper != null) + " !isListenerCreated : "+!isListenerCreated +" helper.isListening() :" +helper.isListening());
        if (helper != null && !isListenerCreated && !helper.isListening()) {
            helper.startListening(cryptoObject);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        Log.i(TAG, " on pause ");
        if (helper != null) {
            helper.stopListening();
            isListenerCreated = false;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.i(TAG, " on destroy ");
        if (helper != null) {
            helper.stopListening();
        }
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);


        if (requestCode == MainActivity.REQUEST_CODE) {
            if (resultCode == RESULT_OK) {

                Intent mIntent = new Intent();
                mIntent.putExtra("LOGOUT", "LOGOUT");
                setResult(Activity.RESULT_OK, mIntent);
                finish();
            }
        }
    }
    /**
     * Open the security setting
     * @param view
     */
    public void gotoSettings(View view) {
        Intent mIntent = new Intent(Settings.ACTION_SECURITY_SETTINGS);
        startActivity(mIntent);
    }

    /**
     * Restart the current intent
     * @param view
     */
    public void restart(View view) {
        Intent mIntent = getIntent();
        finish();
        startActivity(mIntent);
    }

    private void generateKey() throws FingerprintException {
        try {

            keyStore = KeyStore.getInstance("AndroidKeyStore");

            keyGenerator = KeyGenerator.getInstance(KeyProperties.KEY_ALGORITHM_AES, "AndroidKeyStore");

            keyStore.load(null);
            keyGenerator.init(new
                    KeyGenParameterSpec.Builder(DEFAULT_KEY_NAME,
                    KeyProperties.PURPOSE_ENCRYPT |
                            KeyProperties.PURPOSE_DECRYPT)
                    .setBlockModes(KeyProperties.BLOCK_MODE_CBC)
                    .setUserAuthenticationRequired(true)
                    .setEncryptionPaddings(
                            KeyProperties.ENCRYPTION_PADDING_PKCS7)
                    .build());

            keyGenerator.generateKey();

        } catch (KeyStoreException
                | NoSuchAlgorithmException
                | NoSuchProviderException
                | InvalidAlgorithmParameterException
                | CertificateException
                | IOException exc) {
            exc.printStackTrace();
            throw new FingerprintException(exc);
        }
    }

    public boolean initCipher() {
        try {
            cipher = Cipher.getInstance(
                    KeyProperties.KEY_ALGORITHM_AES + "/"
                            + KeyProperties.BLOCK_MODE_CBC + "/"
                            + KeyProperties.ENCRYPTION_PADDING_PKCS7);
        } catch (NoSuchAlgorithmException |
                NoSuchPaddingException e) {
            throw new RuntimeException("Failed to get Cipher", e);
        }

        try {
            keyStore.load(null);
            SecretKey key = (SecretKey) keyStore.getKey(DEFAULT_KEY_NAME,
                    null);
            cipher.init(Cipher.ENCRYPT_MODE, key);
            return true;
        } catch (KeyPermanentlyInvalidatedException e) {
            return false;
        } catch (KeyStoreException | CertificateException
                | UnrecoverableKeyException | IOException
                | NoSuchAlgorithmException | InvalidKeyException e) {
            throw new RuntimeException("Failed to init Cipher", e);
        }
    }

    private class FingerprintException extends Exception {

        public FingerprintException(Exception e) {
            super(e);
        }
    }

    /**
     * This method will send the analytics to the web
     */
    public void postAnalytics() {
        TOTAL_TIME_TAKEN = System.currentTimeMillis() - timeStarted;
        new PostAnalytics().execute();
    }

    /**
     * This private class will post the analytics to the analytics web-page
     */
    private class PostAnalytics extends AsyncTask<Void, Void, String> {

        String postData;
        URL url;
        HttpsURLConnection connection;
        OutputStreamWriter outputStreamWriter;
        BufferedReader bufferedReader;

        @Override
        protected String doInBackground(Void... arg) {

            long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);
            try {

                String analytics_url = getString(R.string.domain_name) + getString(R.string.ANALYTICS_POST_URL);
                url = new URL(analytics_url);

                postData = URLEncoder.encode("name", "UTF-8")
                        + "=" + URLEncoder.encode("fingerprint", "UTF-8") +"&"
                        +  URLEncoder.encode("version", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.VERSION, "UTF-8") +"&"
                        +  URLEncoder.encode("sessionID", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.getSessionID_String(sessionID), "UTF-8") +"&"
                        +  URLEncoder.encode("jobtitle", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.jobTitle, "UTF-8") +"&"
                        +  URLEncoder.encode("affiliation", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.affiliation_field, "UTF-8") +"&"
                        +  URLEncoder.encode("timeTaken", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(TOTAL_TIME_TAKEN), "UTF-8") +"&"
                        +  URLEncoder.encode("firstAction", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(firstActionTime), "UTF-8") +"&"
                        +  URLEncoder.encode("totalClicks", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(TOTAL_CLICKS), "UTF-8") +"&"
                        +  URLEncoder.encode("metrics", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(numSuccessAuthentication), "UTF-8")
                            +","+URLEncoder.encode(Integer.toString(numFailedAuthentication), "UTF-8") +"&"
                        +  URLEncoder.encode("metricInits", "UTF-8")
                        + "=" + URLEncoder.encode("Succ", "UTF-8")  +","+ URLEncoder.encode("Fail", "UTF-8");

                Log.i(TAG, "In url parse data: " + postData);

                connection = (HttpsURLConnection) url.openConnection();
                connection.setRequestMethod("POST");
                connection.setDoInput(true);
                connection.setDoOutput(true);
                connection.connect();
                connection.getOutputStream().write(postData.getBytes());
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getInputStream()));

                String line = null;
                String data = "";

                while ((line = bufferedReader.readLine()) != null) {
                    data += line;
                }


                Log.i(TAG, "retrieved data is "+data);
                return data;
            } catch (Exception e) {
                Log.i(TAG, "Exception " + e.getMessage());
                return new String("Exception: " + e.getMessage());
            }
        }

        /*Reset all the analytics values */
        @Override
        protected void onPostExecute(String result) {
            Log.i(TAG, "In onPostExecute with result: " + result);
            //reset all the values
            timeStarted = 0;
            TOTAL_TIME_TAKEN = 0;
            firstActionTime = 0;
            timeForTheMostCurrentAction = 0;
            numFailedAuthentication = 0;
            numSuccessAuthentication = 0;
            TOTAL_CLICKS = 0;
            isFirstAction = true;
        }

    }

}
