/**
 * CMSC435 Group Project “Authentication”
 * For this app we used the following resources
 *  :https://developer.android.com/samples/FingerprintDialog/index.html
 *  :https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/
 */

/**
 * This activity will be called when a user logged in successfully to any of the authentication systems.
 * If the user tested all the authentication systems, then this screen will have a button that let
 * the user take a survey about the authentication system experience. If the user didn't test all
 * the authentication systems, then this activity will redirect the user to the mainActivity.
 */

package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class LoggedInActivity extends AppCompatActivity {

    private Button survey_button, logout_button;

    //a flag that indicants if all the authentication systems have been used
    private boolean completed = false;

    private final String COMPLETED = "COMPLETED";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.loggedin_layout);

        if (savedInstanceState != null) {
            completed = savedInstanceState.getBoolean(COMPLETED);
        }

        survey_button = (Button) findViewById(R.id.survey_button);
        logout_button = (Button) findViewById(R.id.logout_button);

        //Checks to see if the current authentication system is the last one
        if (getIntent().getExtras().getBoolean(MainActivity.IS_LAST)){

            //If it is the last one, then make the continue button visible
            logout_button.setVisibility(View.GONE);
        } else {

            //If it is NOT the las one, then display this screen for 1 second and
            //Go back to the MainActivity
            survey_button.setVisibility(View.GONE);
            logout_button.setVisibility(View.GONE);

            Handler handler = new Handler();

            handler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    Intent mIntent = new Intent();
                    mIntent.putExtra("LOGOUT", "LOGOUT");
                    setResult(Activity.RESULT_OK, mIntent);
                    finish();
                }
            }, 1000);
        }


    }


    /**
     * When this method is called, both the survey and logout_button will ge invisible
     */
    public void logout(View view) {
        survey_button.setVisibility(View.GONE);
        logout_button.setVisibility(View.GONE);
    }

    /**
     * This method will be called when the user clicked on the “continue”;
     * in which case, this method will redirect the user to the survey web-page.
     */
    public void takeSurvey(View view) {
        completed = true;

        //make the "continue" button invisible
        view.setVisibility(View.GONE);
        survey_button.setVisibility(View.GONE);

        //mark which authentication systems are used during this session
        String type1 = "null";
        String type2 = "null";
        String type3 = "null";
        for (int i=0; i < MainActivity.survey_apps.size(); i++) {
            if (MainActivity.survey_apps.get(i).equals("FingerPrintAuth")) {
                type2 = "fingerprint";
            } else if (MainActivity.survey_apps.get(i).equals("RegularPasswordAuth")) {
                type1 = "password";
            } else if (MainActivity.survey_apps.get(i).equals("YubikeyAuth")) {
                type3 = "yubikey";
            }
        }

        //clear the survey_apps arrayList
        MainActivity.survey_apps.clear();

        //Get the current sessionID
        long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

        String survey_url = getString(R.string.domain_name) + getString(R.string.SURVEY_LINK);

        //redirect the user to the survey page, by posting which type of authentication
        // systems the survey questions should be about
        String surveyExtraInputs = "type1=" + type1 + "&type2=" + type2 + "&type3=" + type3
                +"&sessionID=" + MainActivity.getSessionID_String(sessionID);
        Intent goToSurveyPage = new Intent(Intent.ACTION_VIEW, Uri.parse(survey_url + surveyExtraInputs));
        logout_button.setVisibility(View.VISIBLE);
        startActivity(goToSurveyPage);
        logout(view);

    }


    @Override
    public void onRestart() {
        super.onRestart();

        //If everything is completed, display a Thank you text on the screen for 1 second
        //and close the application
        if (completed) {
            TextView success_label = (TextView) findViewById(R.id.success_label);
            success_label.setText("Thank You!");

            Handler handler = new Handler();

            handler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    Intent mIntent = new Intent();
                    mIntent.putExtra("LOGOUT", "LOGOUT");
                    setResult(Activity.RESULT_OK, mIntent);
                    finish();
                }
            }, 1000);
        }
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {

        savedInstanceState.putBoolean(COMPLETED, completed);
        super.onSaveInstanceState(savedInstanceState);
    }

    @Override
    public void onResume() {
        super.onResume();
    }

}
