/**
 * CMSC435 Group Project “Authentication”
 * For this app we used the following resources
 *  :https://developer.android.com/samples/FingerprintDialog/index.html
 *  :https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/
 *  :https://github.com/nadam/nfc-reader
 */

/**
 * This Activity is the main start of this application that calls two or many authentication
 * activities and starts them. It also makes sure each authentication activity is run only one time.
 * At the start of this activity, the user will be required to create a temporary user name and
 * password to test out the password login system.
 */

package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.Manifest;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.hardware.fingerprint.FingerprintManager;
import android.nfc.NfcAdapter;
import android.os.AsyncTask;
import android.os.Build;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.net.ssl.HttpsURLConnection;

import static java.lang.Thread.sleep;

public class MainActivity extends AppCompatActivity {

    //Version of this app
    public static final String VERSION = "2.0.3.1";

    //holds the current sessionID
    private long sessionID;

    //The following variables are used for a key-value map to store and transfer data across activities
    public static final String USER_NAME_KEY = "userName";
    public static final String PASSWORD_KEY = "password";
    public static final String IS_LAST = "isLast";
    public static final String JOB_TITLE_KEY = "jobKey";
    public static final String AFFILI_KEY = "affiliKey";
    public static final String SESSION_ID = "sessionID";

    private static final String APP_LIST_KEY = "appList";
    private static final String SURVEY_LIST_KEY = "surveyList";
    private static final String IS_FIRST_TIME_KEY = "isFirstTimeUse";

    public static final int REQUEST_CODE = 3;

    //private TextView error_message;
    private Button start_button;
    private EditText userNameValue, passwordValue, confirmPassValue, job_title;

    /*This list will hold all the authentication system names,
    and one of them will be selected at random and start. */
    private ArrayList<String> app_list = new ArrayList<String>();

    //This list will hold all authentication system names for the survey part
    public static ArrayList<String> survey_apps = new ArrayList<String>();

    //Holds  username and password
    private String userNameStr, passWordStr;

    //Holds the value for the job title and affiliation
    //If this two values are not reset, then a string dropped will be sent to analytics
    public static String jobTitle = "dropped";
    public static String affiliation_field = "dropped";

    //This flag will indicate if this activity is being used for the first time.
    //If the user successfully created a user name and password, then this flag will be set to false.
    private boolean isFirstTimeUse = true;


    //Used for testing
    static private final String TAG = "MainActivity";

    private Spinner list_affiliation;
    private static final String[]affiliation = {"Customer", "Supplier", "Dealer", "Employee", "No affiliation"};

    /*The following variables hold analytics data */

    //Total time taken
    private static long totalTimeTaken;

    //The time this activity got started.
    private static long timeStarted;

    //The time from the start to the first action
    public static long firstActionTime;

    private static boolean isFirstAction = true;

    //The number of times the help button is clicked
    private static int num_of_help_click = 0;

    //The number of times start button was clicked
    private static int num_of_start_click =0;

    //The number of failed authentication
    private static int numFailedAuthentication;

    //The number of successful authentication
    private static int numSuccessAuthentication;



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);


        //Create a drop-down menu for "affiliation" question
        list_affiliation = (Spinner) findViewById(R.id.spinner);

        ArrayAdapter<String> adapter = new ArrayAdapter<String>(MainActivity.this,
                android.R.layout.simple_spinner_item, affiliation);

        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        list_affiliation.setAdapter(adapter);
        list_affiliation.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view,
                                       int position, long id) {
                affiliation_field = (String) parent.getItemAtPosition(position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        //Get the reference of the user inputs
        userNameValue = (EditText) findViewById(R.id.userNameValue);
        passwordValue = (EditText) findViewById(R.id.passwordValue);
        confirmPassValue = (EditText) findViewById(R.id.confirm_pass);
        start_button = (Button) findViewById(R.id.start_button);
        job_title = (EditText) findViewById(R.id.job_title);

        //If this activity is not created, then retrieve the values of the saved variables
        //Otherwise, add any authentication system name to the app_list
        if (savedInstanceState != null) {
            survey_apps = (ArrayList<String>) savedInstanceState.getStringArrayList(SURVEY_LIST_KEY);
            app_list = (ArrayList<String>) savedInstanceState.getStringArrayList(APP_LIST_KEY);
            userNameStr = savedInstanceState.getString(USER_NAME_KEY);
            passWordStr = savedInstanceState.getString(PASSWORD_KEY);
            isFirstTimeUse = savedInstanceState.getBoolean(IS_FIRST_TIME_KEY);
            jobTitle = savedInstanceState.getString(JOB_TITLE_KEY);
            affiliation_field = savedInstanceState.getString(AFFILI_KEY);
            sessionID = savedInstanceState.getLong(SESSION_ID);

        } else {

            //create a new sessionID
            sessionID = System.currentTimeMillis();

            //Save the time this activity got started
            timeStarted = System.currentTimeMillis();

            /*              Fingerprint Authentication
            * Check if the device has a fingerprint hardware
            * Also, Make sure the SDK version is greater than 22
            */
            FingerprintManager fingerprintManager = getSystemService(FingerprintManager.class);
            if ((Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) &&
                    (ActivityCompat.checkSelfPermission(this, Manifest.permission.USE_FINGERPRINT)
                            == PackageManager.PERMISSION_GRANTED)
                    &&(fingerprintManager.isHardwareDetected())) {

                //If everything is satisfactory, then add fingerprint to the app_list
                app_list.add("FingerPrintAuth");

                //add the fingerprint to the survey list
                survey_apps.add("FingerPrintAuth");
            }

            //Add the RegularPasswordAuthentication to the app_list
            app_list.add("RegularPasswordAuth");

            //add the regular password to the survey list
            survey_apps.add("RegularPasswordAuth");

        }


        //If there is no Intent in app_list, then tell the user that their device doesn't support our app.
        //Then close the app after 2 second
        if (app_list.isEmpty()) {
            start_button.setVisibility(View.GONE);
            //error_message.setText(R.string.low_sdk_version_mess);
            Toast.makeText(getApplicationContext(), R.string.low_sdk_version_mess,
                    Toast.LENGTH_LONG).show();
            try {
                sleep(2000);
            } catch (Exception e) {
                e.printStackTrace();
            }
            finish();
        }
    }

    public void onItemSelected(AdapterView<?> parent, View v, int position, long id) {}

    //This method is called when the start button is clicked
    public void start(View view) {

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(isFirstAction) {
            isFirstAction = false;
            firstActionTime = System.currentTimeMillis() - timeStarted;
        }

        //increment the number of times the user clicked the start button
        num_of_start_click++;

        //If the user has not created an account yet, then make sure to check the user's input for correctness
        if (isFirstTimeUse) {

            //check if the user entered the user name
            if (TextUtils.isEmpty(userNameValue.getText().toString())) {
                Toast.makeText(getApplicationContext(), "Please Enter A User Name",
                        Toast.LENGTH_LONG).show();

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            } else if (TextUtils.isEmpty(passwordValue.getText().toString())) {
                Toast.makeText(getApplicationContext(), "Please Enter A Password",
                        Toast.LENGTH_LONG).show();

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            } else if (TextUtils.isEmpty(confirmPassValue.getText().toString())) {
                Toast.makeText(getApplicationContext(), "Please Enter Confirm Password",
                        Toast.LENGTH_LONG).show();

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            } else if (TextUtils.isEmpty(job_title.getText().toString())) {
                Toast.makeText(getApplicationContext(), "Please Enter Job Title",
                        Toast.LENGTH_LONG).show();

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            }

            //Get the username from the text field
            userNameStr = userNameValue.getText().toString();

            //get the password value from the text field
            passWordStr = passwordValue.getText().toString();
            String confirmPassStr  = confirmPassValue.getText().toString();


            //check if the password is strong
            if (!isPasswordStrong(passWordStr)) {

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            } else if (!passWordStr.equals(confirmPassStr)) {
                Toast.makeText(getApplicationContext(), "Password and Confirm Password Must Match",
                        Toast.LENGTH_LONG).show();

                //increment the number of failed attempt and don't let the user go to the next screen
                numFailedAuthentication++;
                return;
            }

            //get the job title from the text field
            jobTitle = job_title.getText().toString();


            //Indicate the user has successfully created an account
            isFirstTimeUse = false;

            //increment the number of times the user has successfully created an account
            numSuccessAuthentication++;

            RadioGroup yubikeyQuestionGroup = (RadioGroup) findViewById(R.id.yubikey_radio_G);
            int checkedID = yubikeyQuestionGroup.getCheckedRadioButtonId();
            RadioButton yubikeyQuestionButton = (RadioButton) findViewById(checkedID);
            /*
            * Check if the device has NFC system
            * If it has one, and if the user has yubikey, then add TagViewer to the app_list
            * */
            if ((NfcAdapter.getDefaultAdapter(this) != null)
                    && !yubikeyQuestionButton.getText().toString().equals("No")) {
                app_list.add("YubikeyAuth");

                //add the yubikey to the survey
                survey_apps.add("YubikeyAuth");
            }
        }

        //Exit the app if the app_list is empty
        //Else pick an activity and start it.
        if (app_list.isEmpty()) {
            finish();
        } else {
            //Randomly pick an activity and start it.
            //int randomNum = new Random().nextInt(app_list.size());
            int randomNum = app_list.size() - 1;
            String randomlySelectedAuth = app_list.remove(randomNum);

            startActivityForResult(getIntentFromName(randomlySelectedAuth), REQUEST_CODE);

        }
    }

    /**
     * This method gets executed when the user clicks on the help button for the password.
     *This method also will display a dialog window that tells the user what kind of password they should create.
     */
    public void password_help(View view) {

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(isFirstAction) {
            isFirstAction = false;
            firstActionTime = System.currentTimeMillis() - timeStarted;
        }

        //increment the number of times the user clicked the help button
        num_of_help_click++;

        //Create an alert dialog and show the password requirements
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.password_requirements_title);
        builder.setMessage(R.string.password_requirements_message);
        builder.setPositiveButton("OK",
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {}
                });
        builder.create().show();
    }


    /**
     * If this activity restarts and if there is no authentication activity to run, then exit the app.
     */
    @Override
    public void onRestart() {
        super.onRestart();
        Log.i(TAG, "app_list.isEmpty() "+app_list.isEmpty()+"  !isFirstTimeUse "+!isFirstTimeUse);
        if (app_list.isEmpty() && !isFirstTimeUse) {
            finish();
        }
    }

    /**
     * When this activity is finished, post the analytics to the analytics web-page
     */
    @Override
    public void onDestroy() {

        totalTimeTaken = System.currentTimeMillis() - timeStarted;
        new PostAnalytics().execute();

        super.onDestroy();
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {

        savedInstanceState.putStringArrayList(APP_LIST_KEY, app_list);
        savedInstanceState.putStringArrayList(SURVEY_LIST_KEY, survey_apps);
        savedInstanceState.putString(USER_NAME_KEY, userNameStr);
        savedInstanceState.putString(PASSWORD_KEY, passWordStr);
        savedInstanceState.putBoolean(IS_FIRST_TIME_KEY, isFirstTimeUse);
        savedInstanceState.putString(JOB_TITLE_KEY, jobTitle);
        savedInstanceState.putString(AFFILI_KEY, affiliation_field);
        savedInstanceState.putLong(SESSION_ID, sessionID);
        super.onSaveInstanceState(savedInstanceState);
    }

    /**
     * After each activity is finished running, start another activity; or exit the app if all activities are run at most once.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == REQUEST_CODE) {
            if (resultCode == RESULT_OK) {
                if (app_list.isEmpty()) {
                    finish();
                } else {
                    //Pick unused activity and start it.
                    int randomNum = app_list.size() - 1;
                    String randomlySelectedAuth = app_list.remove(randomNum);
                    startActivityForResult(getIntentFromName(randomlySelectedAuth), REQUEST_CODE);
                }
            }
        }
    }

    /**
     * This method will return an Intent for a particular authentication activity
     * @param AuthName a name of the authentication system
     * @return Intent
     */
    private Intent getIntentFromName(String AuthName) {
        Intent mIntent = null;
        if (AuthName.equals("FingerPrintAuth")) {
            mIntent = new Intent(MainActivity.this, FingerPrintAuth.class);

        } else if (AuthName.equals("RegularPasswordAuth")) {

            //Pass the user name and password for the regular password authentication system
            mIntent = new Intent(MainActivity.this, RegularPasswordActivity.class);
            mIntent.putExtra(USER_NAME_KEY, userNameStr);
            mIntent.putExtra(PASSWORD_KEY, passWordStr);
        } else if (AuthName.equals("YubikeyAuth")) {

            mIntent = new Intent(MainActivity.this, TagViewer.class);
        }

        mIntent.putExtra(IS_LAST, app_list.isEmpty());
        mIntent.putExtra(SESSION_ID, sessionID);

        return mIntent;
    }

    /**
     * This method will check if the password is strong
     * @param password
     * @return boolean true if the password is strong
     */
    public boolean isPasswordStrong(String password) {

        if (password.length() < 8) {
            Toast.makeText(getApplicationContext(), "The password must be at least 8 characters long ",
                    Toast.LENGTH_LONG).show();
            return false;
        } else if (password.equals(password.toLowerCase()) || password.equals(password.toUpperCase())) {
            Toast.makeText(getApplicationContext(), "The password must contain both upper and lower case letters",
                    Toast.LENGTH_LONG).show();
            return false;
        } else if (!password.matches(".*\\d+.*")) {
            Toast.makeText(getApplicationContext(), "The password must contain at least one number ",
                    Toast.LENGTH_LONG).show();
            return false;
        }

        Pattern pattern = Pattern.compile("[^a-zA-Z0-9 ]", Pattern.CASE_INSENSITIVE);
        Matcher matcher = pattern.matcher(password);

        if (!matcher.find()) {
            Toast.makeText(getApplicationContext(), "The password must contain at least one special character ",
                    Toast.LENGTH_LONG).show();
            return false;
        }

        pattern = Pattern.compile("\\d{3,}", Pattern.CASE_INSENSITIVE);
        matcher = pattern.matcher(password);

        if (matcher.find()) {
            Toast.makeText(getApplicationContext(), "The password should not have no more than two consecutive numbers ",
                    Toast.LENGTH_LONG).show();
            return false;
        }

        return true;
    }


    //Convert System.currentTimeMillis to "Phone_MMM_dd_yyy_HH_mm" format
    //so, the value can be transmitted to analytics web-page.
    public static String getSessionID_String(long sessionID_Val) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("MMM_dd_yyy_HH_mm");
        Date currentTimeToDate = new Date(sessionID_Val);
        return "Phone_"+simpleDateFormat.format(currentTimeToDate);
    }

    /**
     * This private class will post the analytics to the analytics web-page
     */
    private class PostAnalytics extends AsyncTask<Void, Void, String> {

        String postData;
        URL url;
        HttpsURLConnection connection;
        OutputStreamWriter outputStreamWriter;
        BufferedReader bufferedReader;

        @Override
        protected String doInBackground(Void... arg) {


            //The number of times the user clicked a button on this screen
            int total_clicks = num_of_start_click + num_of_help_click;

            try {
                String analytics_url = getString(R.string.domain_name) + getString(R.string.ANALYTICS_POST_URL);
                url = new URL(analytics_url);

                postData = URLEncoder.encode("name", "UTF-8")
                        + "=" + URLEncoder.encode("register_Phone", "UTF-8") +"&"
                        +  URLEncoder.encode("version", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.VERSION, "UTF-8") +"&"
                        +  URLEncoder.encode("sessionID", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.getSessionID_String(sessionID), "UTF-8") +"&"
                        +  URLEncoder.encode("jobtitle", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.jobTitle, "UTF-8") +"&"
                        +  URLEncoder.encode("affiliation", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.affiliation_field, "UTF-8") +"&"
                        +  URLEncoder.encode("timeTaken", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(totalTimeTaken), "UTF-8") +"&"
                        +  URLEncoder.encode("firstAction", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(firstActionTime), "UTF-8") +"&"
                        +  URLEncoder.encode("totalClicks", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(total_clicks), "UTF-8") +"&"
                        +  URLEncoder.encode("metrics", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(numSuccessAuthentication), "UTF-8")
                        +","+URLEncoder.encode(Integer.toString(numFailedAuthentication), "UTF-8")
                        +","+URLEncoder.encode(Integer.toString(num_of_help_click), "UTF-8")+"&"
                        +  URLEncoder.encode("metricInits", "UTF-8")
                        + "=" + URLEncoder.encode("Succ", "UTF-8") +"," +URLEncoder.encode("fail", "UTF-8")
                        +"," +URLEncoder.encode("num_help", "UTF-8");

                Log.i(TAG, "In url parse data: " + postData);

                connection = (HttpsURLConnection) url.openConnection();
                connection.setRequestMethod("POST");
                connection.setDoInput(true);
                connection.setDoOutput(true);
                connection.connect();
                connection.getOutputStream().write(postData.getBytes());
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getInputStream()));

                String line = null;
                String data = "";

                while ((line = bufferedReader.readLine()) != null) {
                    data += line;
                }


                Log.i(TAG, "retrieved data is "+data);
                return data;
            } catch (Exception e) {
                Log.i(TAG, "Exception " + e.getMessage());
                return new String("Exception: " + e.getMessage());
            }
        }

        /*Reset all the analytics values */
        @Override
        protected void onPostExecute(String result) {
            Log.i(TAG, "In onPostExecute with result: " + result);
            //reset all values
            totalTimeTaken = 0;
            timeStarted = 0;
            firstActionTime = 0;
            num_of_start_click = 0;
            num_of_help_click = 0;
            numFailedAuthentication = 0;
            numSuccessAuthentication = 0;
            isFirstAction = true;
            sessionID = System.currentTimeMillis();
        }

    }

}
