/**
 * CMSC435 Group Project “Authentication”
 * For this app we used the following resources
 *  :https://developer.android.com/samples/FingerprintDialog/index.html
 *  :https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/
 */

/**
 * This activity represents a regular password login system. The user will be asked to input
 * a user name and a password, which they created at the start of application, to login.
 */

package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLEncoder;

import javax.net.ssl.HttpsURLConnection;

public class RegularPasswordActivity  extends AppCompatActivity {

    private static final String TAG = "RegularPassActivity ";

    //Total time taken
    private static long totalTimeTaken;

    //The time this activity got started.
    private static long timeStarted;

    //The time from the start to the first action
    public static long firstActionTime;

    private static boolean isFirstAction = true;

    //number of clicks
    private static int total_clicks = 0;

    //The number of failed authentication
    private static int numFailedAuthentication;

    //The number of successful authentication
    private static int numSuccessAuthentication;

    private EditText user_name, password;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.regular_password_layout);

        //save the time this activity got started
        timeStarted = System.currentTimeMillis();

        user_name = (EditText) findViewById(R.id.user_name);
        password = (EditText) findViewById(R.id.password);
    }

    /**
     * This method will make sure that the password and the username matched with the user name
     * and password that was created at the start of this app. If the login is a success,
     * then the user will be redirected to a login success screen
     */
    public void login(View view){

        total_clicks++;

        Intent mIntent = getIntent();
        String uName = user_name.getText().toString();
        String passW = password.getText().toString();

        String realUName = mIntent.getExtras().getString(MainActivity.USER_NAME_KEY);
        String realPassW = mIntent.getExtras().getString(MainActivity.PASSWORD_KEY);

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(isFirstAction) {
            isFirstAction = false;
            firstActionTime = System.currentTimeMillis() - timeStarted;
        }

        //check if the username and password match
        if (uName.equals(realUName) && passW.equals(realPassW)) {

            //increment the number of successful authentication
            numSuccessAuthentication++;

            //if so, save the total time this activity was alive and post the analytics
            totalTimeTaken = System.currentTimeMillis() - timeStarted;
            new PostAnalytics().execute();

            //get the current sessionID
            long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

            //redirect to the LoggedInActivity
            Intent loggedInActivity = new Intent(RegularPasswordActivity.this, LoggedInActivity.class);
            //startActivity(loggedInActivity);
            loggedInActivity.putExtra(MainActivity.IS_LAST, getIntent().getExtras().getBoolean(MainActivity.IS_LAST));
            loggedInActivity.putExtra(MainActivity.SESSION_ID, sessionID);


            startActivityForResult(loggedInActivity, MainActivity.REQUEST_CODE);

        } else {
            numFailedAuthentication++;
            Toast.makeText(getApplicationContext(), R.string.loginIncorrect,
                    Toast.LENGTH_LONG).show();
        }
    }

    /*
    * This method is called when the user clicks the forgot password link*/
    public void forgotPassword(View view) {

        Intent mIntent = getIntent();

        String realUName = mIntent.getExtras().getString(MainActivity.USER_NAME_KEY);
        String realPassW = mIntent.getExtras().getString(MainActivity.PASSWORD_KEY);

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Your Username and Passwords are:");
        builder.setMessage("Username: "+ realUName+"\nPassword: "+realPassW);
        builder.setPositiveButton("OK",
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {}
                });
        builder.create().show();
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == MainActivity.REQUEST_CODE) {
            if (resultCode == RESULT_OK) {


                Intent mIntent = new Intent();
                mIntent.putExtra("LOGOUT", "LOGOUT");
                setResult(Activity.RESULT_OK, mIntent);

                finish();
            }
        }
    }

    /**
     * This private class will post the analytics to the analytics web-page
     */
    private class PostAnalytics extends AsyncTask<Void, Void, String> {

        String postData;
        URL url;
        HttpsURLConnection connection;
        OutputStreamWriter outputStreamWriter;
        BufferedReader bufferedReader;

        @Override
        protected String doInBackground(Void... arg) {

            long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

            try {
                String analytics_url = getString(R.string.domain_name) + getString(R.string.ANALYTICS_POST_URL);
                url = new URL(analytics_url);

                postData = URLEncoder.encode("name", "UTF-8")
                        + "=" + URLEncoder.encode("password", "UTF-8") +"&"
                        +  URLEncoder.encode("version", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.VERSION, "UTF-8") +"&"
                        +  URLEncoder.encode("sessionID", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.getSessionID_String(sessionID), "UTF-8") +"&"
                        +  URLEncoder.encode("jobtitle", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.jobTitle, "UTF-8") +"&"
                        +  URLEncoder.encode("affiliation", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.affiliation_field, "UTF-8") +"&"
                        +  URLEncoder.encode("timeTaken", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(totalTimeTaken), "UTF-8") +"&"
                        +  URLEncoder.encode("firstAction", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(firstActionTime), "UTF-8") +"&"
                        +  URLEncoder.encode("totalClicks", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(total_clicks), "UTF-8") +"&"
                        +  URLEncoder.encode("metrics", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(numSuccessAuthentication), "UTF-8") +","+URLEncoder.encode(Integer.toString(numFailedAuthentication), "UTF-8") +"&"
                        +  URLEncoder.encode("metricInits", "UTF-8")
                        + "=" + URLEncoder.encode("Succ", "UTF-8") +"," +URLEncoder.encode("Fail", "UTF-8");

                Log.i(TAG, "In url parse data: " + postData);

                connection = (HttpsURLConnection) url.openConnection();
                connection.setRequestMethod("POST");
                connection.setDoInput(true);
                connection.setDoOutput(true);
                connection.connect();
                connection.getOutputStream().write(postData.getBytes());
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getInputStream()));

                String line = null;
                String data = "";

                while ((line = bufferedReader.readLine()) != null) {
                    data += line;
                }


                Log.i(TAG, "retrieved data is "+data);
                return data;
            } catch (Exception e) {
                Log.i(TAG, "Exception " + e.getMessage());
                return new String("Exception: " + e.getMessage());
            }
        }

        /*Reset all the analytics values */
        @Override
        protected void onPostExecute(String result) {
            Log.i(TAG, "In onPostExecute with result: " + result);
            //reset all the values
            totalTimeTaken = 0;
            timeStarted = 0;
            firstActionTime = 0;
            total_clicks = 0;
            numFailedAuthentication = 0;
            numSuccessAuthentication = 0;
            isFirstAction = true;;
        }

    }
}
