/**
 * CMSC435 Group Project “Authentication”
 * For this app we used the following resources
 *  :https://developer.android.com/samples/FingerprintDialog/index.html
 *  :https://www.androidauthority.com/how-to-add-fingerprint-authentication-to-your-android-app-747304/
 *  :https://github.com/nadam/nfc-reader
 */

package com.caterpillarEffect.cmsc435.authentication_prototypes;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.PendingIntent;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.nfc.NfcAdapter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Parcelable;
import android.provider.Settings;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLEncoder;

import javax.net.ssl.HttpsURLConnection;

/**
 * An {@link Activity} which handles a broadcast of a new tag that the device just discovered.
 */
public class TagViewer extends Activity {

    private static final String TAG = "TagViewer ";

    private NfcAdapter mAdapter;
    private PendingIntent mPendingIntent;

    private AlertDialog mDialog;

    //For the analytics
    //Total time taken
    private static long totalTimeTaken;

    //The time this activity got started.
    private static long timeStarted;

    //The time from the start to the first action
    public static long firstActionTime;

    private static boolean isFirstAction = true;

    //number of clicks
    private static int total_clicks = 0;

    //The number of failed authentication
    private static int numFailedAuthentication;

    //The number of successful authentication
    private static int numSuccessAuthentication;


    private static boolean isLast;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.tag_viewer);
        resolveIntent(getIntent());

        mDialog = new AlertDialog.Builder(this).setNeutralButton("Ok", null).create();

        mAdapter = NfcAdapter.getDefaultAdapter(this);
        if (mAdapter == null) {
            showMessage(R.string.error, R.string.no_nfc);
            finish();
            return;
        }

        mPendingIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);

        //save the time this authentication system got started
        timeStarted = System.currentTimeMillis();


        isLast = getIntent().getExtras().getBoolean(MainActivity.IS_LAST);
    }

    private void showMessage(int title, int message) {
        mDialog.setTitle(title);
        mDialog.setMessage(getText(message));
        mDialog.show();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (mAdapter != null) {
            if (!mAdapter.isEnabled()) {
                showWirelessSettingsDialog();
            }
            mAdapter.enableForegroundDispatch(this, mPendingIntent, null, null);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (mAdapter != null) {
            mAdapter.disableForegroundDispatch(this);
        }
    }

    private void showWirelessSettingsDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage(R.string.nfc_disabled);
        builder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialogInterface, int i) {
                Intent intent = new Intent(Settings.ACTION_WIRELESS_SETTINGS);
                startActivity(intent);
            }
        });
        builder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialogInterface, int i) {
                finish();
            }
        });
        builder.create().show();
        return;
    }

    private void resolveIntent(Intent intent) {
        String action = intent.getAction();
        if (NfcAdapter.ACTION_TAG_DISCOVERED.equals(action)
                || NfcAdapter.ACTION_TECH_DISCOVERED.equals(action)
                || NfcAdapter.ACTION_NDEF_DISCOVERED.equals(action)) {
            Parcelable[] rawMsgs = intent.getParcelableArrayExtra(NfcAdapter.EXTRA_NDEF_MESSAGES);
            //NdefMessage[] msgs;
            Context context = getApplicationContext();
            CharSequence text = "Authenticated!";
            int duration = Toast.LENGTH_SHORT;
            Toast toast = Toast.makeText(context, text, duration);
            toast.show();


            //increment the number of this this authentication system was logged in successfully
            numSuccessAuthentication++;

            //If this action is the first action the user took,
            //then save the time the user took to take this action
            if(isFirstAction) {
                isFirstAction = false;
                firstActionTime = System.currentTimeMillis() - timeStarted;
            }

            //save the total time saved
            totalTimeTaken = System.currentTimeMillis() - timeStarted;

            //post the analytics
            new PostAnalytics().execute();

            //get the current sessionID
            long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

            //-----On success, open logged-in screen
            Intent loggedInActivity = new Intent(TagViewer.this, LoggedInActivity.class);
            loggedInActivity.putExtra(MainActivity.IS_LAST, isLast);
            loggedInActivity.putExtra(MainActivity.SESSION_ID, sessionID);
            startActivityForResult(loggedInActivity, MainActivity.REQUEST_CODE);
            //----------
        }
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == MainActivity.REQUEST_CODE) {
            if (resultCode == RESULT_OK) {

                Intent mIntent = new Intent();
                mIntent.putExtra("LOGOUT", "LOGOUT");
                setResult(Activity.RESULT_OK, mIntent);

                finish();
            }
        }
    }
    @Override
    public void onNewIntent(Intent intent) {
        setIntent(intent);
        resolveIntent(intent);
    }

    /*This method is called when skip button is clicked*/
    public void skip(View view) {
        total_clicks++;

        //increment the number of this this authentication system got skipped
        numFailedAuthentication++;

        //If this action is the first action the user took,
        //then save the time the user took to take this action
        if(isFirstAction) {
            isFirstAction = false;
            firstActionTime = System.currentTimeMillis() - timeStarted;
        }

        //save the total time saved
        totalTimeTaken = System.currentTimeMillis() - timeStarted;

        //post the analytics
        new PostAnalytics().execute();

        //close this activity
        Intent mIntent = new Intent();
        mIntent.putExtra("LOGOUT", "LOGOUT");
        setResult(Activity.RESULT_OK, mIntent);

        finish();
    }

    /**
     * This private class will post the analytics to the analytics web-page
     */
    private class PostAnalytics extends AsyncTask<Void, Void, String> {

        String postData;
        URL url;
        HttpsURLConnection connection;
        OutputStreamWriter outputStreamWriter;
        BufferedReader bufferedReader;

        @Override
        protected String doInBackground(Void... arg) {

            long sessionID = getIntent().getExtras().getLong(MainActivity.SESSION_ID);

            try {
                String analytics_url = getString(R.string.domain_name) + getString(R.string.ANALYTICS_POST_URL);
                url = new URL(analytics_url);

                postData = URLEncoder.encode("name", "UTF-8")
                        + "=" + URLEncoder.encode("yubikey", "UTF-8") +"&"
                        +  URLEncoder.encode("version", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.VERSION, "UTF-8") +"&"
                        +  URLEncoder.encode("sessionID", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.getSessionID_String(sessionID), "UTF-8") +"&"
                        +  URLEncoder.encode("jobtitle", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.jobTitle, "UTF-8") +"&"
                        +  URLEncoder.encode("affiliation", "UTF-8")
                        + "=" + URLEncoder.encode(MainActivity.affiliation_field, "UTF-8") +"&"
                        +  URLEncoder.encode("timeTaken", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(totalTimeTaken), "UTF-8") +"&"
                        +  URLEncoder.encode("firstAction", "UTF-8")
                        + "=" + URLEncoder.encode(Long.toString(firstActionTime), "UTF-8") +"&"
                        +  URLEncoder.encode("totalClicks", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(total_clicks), "UTF-8") +"&"
                        +  URLEncoder.encode("metrics", "UTF-8")
                        + "=" + URLEncoder.encode(Integer.toString(numSuccessAuthentication), "UTF-8")
                        +","+URLEncoder.encode(Integer.toString(numFailedAuthentication), "UTF-8") +"&"
                        +  URLEncoder.encode("metricInits", "UTF-8")
                        + "=" + URLEncoder.encode("Succ", "UTF-8") +"," +URLEncoder.encode("skip", "UTF-8");

                Log.i(TAG, "In url parse data: " + postData);

                connection = (HttpsURLConnection) url.openConnection();
                connection.setRequestMethod("POST");
                connection.setDoInput(true);
                connection.setDoOutput(true);
                connection.connect();
                connection.getOutputStream().write(postData.getBytes());
                bufferedReader = new BufferedReader(new InputStreamReader(connection.getInputStream()));

                String line = null;
                String data = "";

                while ((line = bufferedReader.readLine()) != null) {
                    data += line;
                }


                Log.i(TAG, "retrieved data is "+data);
                return data;
            } catch (Exception e) {
                Log.i(TAG, "Exception " + e.getMessage());
                return new String("Exception: " + e.getMessage());
            }
        }

        /*Reset all the analytics values */
        @Override
        protected void onPostExecute(String result) {
            Log.i(TAG, "In onPostExecute with result: " + result);
            //reset all values
            totalTimeTaken = 0;
            timeStarted = 0;
            firstActionTime = 0;
            total_clicks = 0;
            numFailedAuthentication = 0;
            numSuccessAuthentication = 0;
            isFirstAction = true;
        }

    }
}