// Metrics
let time_start = 0;			// Time stamp representing when the session first started
let first_action = 0;		// Time stamp representing when the first action took place
let total_clicks = 0;		// The total number of clicks that have happened
let data = [];				// Array of metrics to be collected
let datainit = [];			// Array of first action metrics to be collected

// Controls
let running = false;		// Whether or not the current session is running. Set to false if you don't want analytics to be collected.

// Config data
let anyl_authCookie;		// name of the authentication cookie we are using
let anyl_seshCookie;		// name of the session cookie we are using (unused)
let anyl_name;				// the name of the authentication method we are collecting metrics for
let anyl_version;			// version of authentication we are using
let anyl_submitPath;		// the path we want to submit data to.

const version = '2.1.6';


function increment_analytic(elementId) {
    if (data[elementId] == null) {
        data[elementId] = 0;
        datainit[elementId] = +new Date() - time_start;
    }
    data[elementId]++;

    // Update the cookie.
    setAnalyticsCookie();
}

// Delete the current cookie
function deleteAnalyticsCookie() {
    const d = new Date();
    d.setTime(d.getTime() + (1 * 24 * 60 * 60 * 1000));
    const expires = `expires=${d.toUTCString()}`;
    document.cookie = `${anyl_authCookie}=;${expires};path=/`;
}

// Set a cookie that contains the sessions analytics.
function setAnalyticsCookie() {
    const d = new Date();
    d.setTime(d.getTime() + (1 * 24 * 60 * 60 * 1000));
    const expires = `expires=${d.toUTCString()}`;

    const val = JSON.stringify(serializeData());
    document.cookie = `${anyl_authCookie}=${val};${expires};path=/`;
}

function getCookieByName(name) {
    const re = new RegExp(`${name}=([^;]+)`);
    const value = re.exec(document.cookie);
    return (value != null) ? unescape(value[1]) : null;
}

// returns the analytics cookie
function getAnalyticsCookie() {
    const name = `${anyl_authCookie}=`;
    const decodedCookie = decodeURIComponent(document.cookie);

    const ca = decodedCookie.split(';');

    let cookieVal = '';

    for (let i = 0; i < ca.length; i++) {
        let c = ca[i];
        while (c.charAt(0) == ' ') {
            c = c.substring(1);
        }
        if (c.indexOf(name) == 0) {
            cookieVal = c.substring(name.length, c.length);
            break;
        }
    }
    return cookieVal;
}

function resetAuthSession() {
    console.log('Resetting session.');
    deleteAnalyticsCookie();
    data = [];
    datainit = [];
    time_start = 0;
    first_action = 0;
    total_clicks = 0;
    running = false;
}


function serializeData() {
    const ret = {};

    ret.name = anyl_name;
    ret.version = anyl_version;
    ret.startTime = time_start;
    ret.timeTaken = (+new Date()) - time_start;
    ret.firstAction = (first_action == 0 || first_action == time_start) ? 0 : first_action - time_start;
    ret.totalClicks = total_clicks;
    ret.metrics = data;
    ret.metricInits = datainit;

    return ret;
}

// Submits the data to a webserver
let request;

function submitSessionData() {
    const host = window.location.origin;

    if (request) request.abort();

    // Serialize our data here.
    const submitData = serializeData();

    submitData.jobtitle = getCookieByName('job_title');
    submitData.affiliation = getCookieByName('affiliation');
    submitData.sessionID = getCookieByName('sessionID');


    request = $.ajax({
        url: (host + anyl_submitPath),
        type: 'post',
        async: false,
        data: submitData,
    });

    request.done((response, textStatus, jqXHR) = > {
        console.log(`Submission response: ${response}`);
})
    ;

    request.fail((jqXHR, textStatus, errorThrown) = > {
        console.error(`The following error occurred: ${textStatus}`, errorThrown);
})
    ;
}

function loadConfigs() {
    // Load in path for our auth cookie
    const ac = $("meta[name='anyl_authCookie']").attr('content');
    const sc = $("meta[name='anyl_sessionCookie']").attr('content');
    const an = $("meta[name='anyl_name']").attr('content');
    const av = $("meta[name='anyl_version']").attr('content');
    const sp = $("meta[name='anyl_submitPath']").attr('content');
    const ra = $("meta[name='anyl_newSession']").attr('content');
    const rae = $("meta[name='anyl_newSession_ifEmpty']").attr('content');

    // Load in the auth cookie
    if (ac != undefined) {
        anyl_authCookie = ac;
        console.log(`The Auth Cookie is set to    ${anyl_authCookie}`);
    } else {
        anyl_authCookie = 'AuthAnalytics';
        console.log(`The Auth Cookie is preset to ${anyl_authCookie}`);
    }

    // Load in path for session cookie
    if (sc != undefined) {
        anyl_sessionCookie = sc;
        console.log(`The Sess Cookie is set to    ${anyl_sessionCookie}`);
    } else {
        anyl_sessionCookie = 'TODO';
        console.log(`The Sess Cookie is preset to ${anyl_sessionCookie}`);
    }

    // Load in the name of our auth type
    if (an != undefined) {
        anyl_name = an;
        console.log(`The Auth name is set to      ${anyl_name}`);
    } else {
        anyl_name = 'undefined';
        console.log(`The Auth name is preset to   ${anyl_name}`);
    }

    // Load in the version
    if (av != undefined) {
        anyl_version = av;
        console.log(`The Auth version is set to    ${anyl_version}`);
    } else {
        anyl_version = '1.0.0';
        console.log(`The Auth version is preset to ${anyl_version}`);
    }

    // load in the submit path for the data
    if (sp != undefined) {
        anyl_submitPath = av;
        console.log(`The Submit Path is set to     ${anyl_submitPath}`);
    } else {
        anyl_submitPath = '/analytics/submitData.php';
        console.log(`The Submit Path is preset to  ${anyl_submitPath}`);
    }


    if (rae != undefined) {
        const cook = getAnalyticsCookie();
        if (cook.length > 0) {
            const jsonData = JSON.parse(cook);

            const name = jsonData.name;

            if (an != name) {
                console.log('Resetting the analytics session because the authentication name is not the same as the one the cookie provided.');
                resetAuthSession();
            }
        }
    }

    // Reset session if we want
    if (ra != undefined) {
        console.log('Meta tag set to force restart the session.');
        resetAuthSession();
    }
}

// Loads in data from session cookie if one is available, otherwise it initializes the default behaviours
function initializeSession() {
    const cook = getAnalyticsCookie();

    // No cookie found
    if (cook == undefined || cook.length == 0) {
        data = [];
        datainit = [];
        time_start = +new Date();
        first_action = 0;
        total_clicks = 0;
        // Cookie found
    } else {
        // Load the cookie data
        const jsonData = JSON.parse(cook);
        console.log('Loaded authentication cookie. Setting values.');

        anyl_name = jsonData.name;
        anyl_version = jsonData.version;
        time_start = jsonData.startTime;
        total_clicks = jsonData.totalClicks;
        first_action = jsonData.firstAction == 0 ? 0 : jsonData.firstAction + time_start;
        data = jsonData.metrics;
        datainit = jsonData.metricInits;

        console.log(`Name: ${anyl_name}`);
        console.log(`Vers: ${anyl_version}`);
        console.log(`Star: ${time_start}`);
        console.log(`totc: ${total_clicks}`);
        console.log(`fact: ${first_action}`);
        console.log(`fact2:${first_action - time_start}`);
    }
}

function createMetricListeners() {
    // Listen to any clicks within the body
    $(document.body).click((event) = > {
        if(
    !event.target.hasAttribute('data-anyl-complete')
)
    {
        if (running) {
            if (first_action == 0) first_action = +new Date();
            total_clicks++;

            if (!event.target.hasAttribute('data-anyl-button-count') && !event.target.hasAttribute('data-anyl-text-count')) {
                setAnalyticsCookie();
            }
        }
    }
})
    ;

    // Document a button press.
    $('[data-anyl-button-count]').click(function () {
        if (running) {
            if (first_action == 0) first_action = +new Date();
            const elementId = Number($(this).data('anyl-button-count'));
            increment_analytic(elementId);
        }
    });

    // Document when a textbox changes.
    $('[data-anyl-text-count]').on('input', function (e) {
        if (running) {
            if (first_action == 0) first_action = +new Date();
            const elementId = Number($(this).data('anyl-text-count'));
            increment_analytic(elementId);
        }
    });

    // Complete the analytical session.
    $('[data-anyl-complete]').click(() = > {
        if(running) {
            if (first_action == 0) first_action = +new Date();
            submitSessionData();
            resetAuthSession();
        }
    }
)
    ;

    running = true;
}

$(document).ready(() = > {
    loadConfigs();
initializeSession();
createMetricListeners();

console.log(`Analytics VERSION: ${version}`);
})
;
