/* Notice that we are setting the function to call when submit is selected */
window.onsubmit = validateForm;

/* This function must return true or false */
/* If true the data will be sent to the server */

/* If false the data will not be sent to the server */
function validateForm() {

    // Checks where the user was prompted to create a PIN for the one-time password authentication method
    var otpPresent = (document.getElementById('otpFields').hasAttribute('hidden') == false);

    // Checks where the user was prompted to create a username and password for the default password authentication method
    var passwordPresent = (document.getElementById('passwordFields').hasAttribute('hidden') == false);


    var pin = null;
    var retypePin = null;

    // if otpPresent, then the volunteer's PIN must be validated
    if (otpPresent) {
        var pin = document.getElementById("pin").value;
        var retypePin = document.getElementById("retypePin").value;
    }


    var userName = null;
    var password = null;
    var retypePassword = null;

    // if passwordPresent, then the volunteer's username and password must be validated
    if (passwordPresent) {
        var userName = document.getElementById("userName").value;
        var password = document.getElementById("password").value;
        var retypePassword = document.getElementById("retypePassword").value;
    }


    // Stores the criteria that the volunteer's credentials have not met
    var invalidMessages = "";

    // Check that the password and retyped password match and that the criteria for creating a password has been met
    if (password !== retypePassword) {
        invalidMessages += "\t- Passwords do not match.\n";
    }
    if (password != null) {
        if (password.length < 8) {
            invalidMessages += "\t- Password must be at least 8 characters long.\n";
        }
        if (/^[^A-Z]+$/.test(password)) {
            invalidMessages += "\t- Password must contain an uppercase character (A-Z).\n";
        }
        if (/^[^a-z]+$/.test(password)) {
            invalidMessages += "\t- Password must contain a lowercase character (a-z).\n"
        }
        if (/^[^0-9]+$/.test(password)) {
            invalidMessages += "\t- Password must contain a digit (0-9).\n";
        }
        if (/^[^@*&;=!.%?_|#$]+$/.test(password)) {
            invalidMessages += "\t- Password must contain a special character\n\t\t(@*&;=!.%?_|#$).\n";
        }
        if (/\d{3,}/.test(password)) {
            invalidMessages += "\t- Password can't contain more than 2 consecutive digits.\n";
        }
    }


    // Check that the PIN and retyped PIN match and that the criteria for making a pin has been met
    if (pin !== retypePin) {
        invalidMessages += "\t- PINs do not match.\n";
    }
    if (pin != null) {
        if (/\D/.test(pin)) {
            invalidMessages += "\t- PIN can only be digits.\n"
        }
        if (pin.length != 4) {
            invalidMessages += "\t- PIN must be 4 digits long.\n";
        }
    }


    // If there is criteria that the volunteer has not met for creating credentials, an alert box appears,
    // and the volunteer is not redirected until they fix the credentials and met all the critera
    if (invalidMessages !== "") {
        alert("Please fix the following information:\n" + invalidMessages);
        return false;
    } else {
        /* Set old cookies to expire */
        setCookie("username", "", -1);
        setCookie("password", "", -1);
        setCookie("rsaKey", "", -1);

        // store the info in cookies for the authentication methods to use

        if (passwordPresent) {
            setCookie("username", userName, 1);
            setCookie("password", password, 1);

        }

        if (otpPresent) {
            setCookie("rsaKey", pin, 1);
        }

        // redirect the page
        return true;
    }

}

function setCookie(cname, cvalue, exdays) {
    var d = new Date();
    d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
    var expires = "expires=" + d.toUTCString();
    document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
}


function getCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') {
            c = c.substring(1);
        }
        if (c.indexOf(name) == 0) {
            return c.substring(name.length, c.length);
        }
    }
    return "";
}

